/* decimal.h */

#ifndef PY_DECIMAL_H
#define PY_DECIMAL_H

#ifdef __cplusplus
extern "C" {
#endif

#define BIG_EXP
    
    
#ifdef BIG_EXP
/* At the moment this has significant speed effect, since
 * there is many exp_t copied */
#define EXP_LIMB_COUNT 5  /* maximal number of limbs per exp */
typedef struct {
    long limbs[EXP_LIMB_COUNT];
    int sign;
    int size;
} exp_t;
#else
#define exp_t long
#endif

#define BASE 10000	/* biggest value of limb power of 10 */
#define LOG 4		/* number of digits per limb LOG = log10(BASE) */
#define LOG_STR "4" /* any ideas how to avoid that? :P */
/* decimalobject struct ******************************************************/

typedef struct {
    PyObject_HEAD
    long ob_size;    /* number of digits */
    unsigned int sign;     /* see sign contstants above */
    exp_t exp;
/*    signed char *digits;*/   /* digits are stored as the actual numbers 0-9 */
    long limb_count;	/* number of limbs */
    long *limbs;
} PyDecimalObject;

static PyTypeObject PyDecimal_DecimalType;

/* contextobject struct ******************************************************/

typedef struct {
    PyObject_HEAD
    long prec;

    /* option flags */
    int rounding;     /* see rounding constants above */
    int rounding_dec; /* dito */
    int capitals;     /* print "e" or "E" before exponents */
    int clamp;        /* change exponents if they are too high */
    
    /* min/max exponents */
    exp_t Emin;
    exp_t Emax;

    /* signal handling -- these are dicts */
    PyObject *traps;   /* if a signal is trapped */
    PyObject *flags;   /* if a signal has occurred, but not been trapped */
    PyObject *ignored; /* if a signal is ignored completely */
} PyDecimalContextObject;

static PyTypeObject PyDecimal_DecimalContextType;

/* functions *****************************************************************/

PyAPI_FUNC(PyObject *) PyDecimal_FromString(char *, long, PyDecimalContextObject *);
PyAPI_FUNC(PyObject *) PyDecimal_FromLong(long);
PyAPI_FUNC(PyObject *) PyDecimal_FromSequence(PyObject *);

/* type checking macros ******************************************************/

#define PyDecimal_Check(op) PyObject_TypeCheck((op), &PyDecimal_DecimalType)
#define PyDecimalContext_Check(op) \
    PyObject_TypeCheck((op), &PyDecimal_DecimalContextType)

/* static constants **********************************************************/

static PyDecimalObject *PyDecimal_NaN;
static PyDecimalObject *PyDecimal_Inf;
static PyDecimalObject *PyDecimal_NegInf;

static PyDecimalContextObject *PyDecimal_DefaultContext;
static PyDecimalContextObject *PyDecimal_BasicContext;
static PyDecimalContextObject *PyDecimal_ExtendedContext;


#ifdef __cplusplus
}
#endif

#endif /* defined(PY_DECIMAL_H) */
